package com.yanzuoguang.mq.dao.impl;

import com.yanzuoguang.dao.impl.BaseDaoImpl;
import com.yanzuoguang.db.DbExecute;
import com.yanzuoguang.mq.dao.QueueServerTokenDao;
import com.yanzuoguang.mq.vo.QueueServerTokenVo;
import com.yanzuoguang.util.vo.MapRow;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 通过Token找到所在服务器
 *
 * @author 颜佐光
 */
@Component
public class QueueServerTokenDaoImpl extends BaseDaoImpl implements QueueServerTokenDao {
    private static final String QUERY_TABLE_SQL = "SHOW TABLES LIKE 'queue_servertoken'";
    private static final String CREATE_TABLE_SQL = "CREATE TABLE `queue_servertoken` (" +
            "  `serverTokenId` varchar(32) NOT NULL COMMENT '编号'," +
            "  `serverId` varchar(32) NOT NULL COMMENT '服务Id'," +
            "  `queueName` varchar(255) NOT NULL DEFAULT '' COMMENT '消息编号'," +
            "  `tokenId` varchar(255) NOT NULL DEFAULT '' COMMENT '标记Id'," +
            "  `tokenVersion` varchar(32) NOT NULL COMMENT '版本号'," +
            "  `updateDate` datetime DEFAULT NULL COMMENT '修改时间'," +
            "  PRIMARY KEY (`serverTokenId`)," +
            "  KEY `IndexNameToken` (`queueName`,`tokenId`)," +
            "  KEY `IndexServerId` (`serverId`)" +
            ") ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='通过Token找到所在服务器'";

    @Override
    protected void init() {
        register(QueueServerTokenVo.class);
    }

    @Override
    protected void initExecute(DbExecute db) {
        List<MapRow> tables = db.query(QueueServerTokenDaoImpl.class, "QUERY_TABLE_SQL", QUERY_TABLE_SQL);
        if (tables.isEmpty()) {
            db.update(QueueServerTokenDaoImpl.class, "CREATE_TABLE_SQL", CREATE_TABLE_SQL);
        }
    }
}
